import unittest
from typing import Callable, Optional

from jivago.lang.registry import Registry, ParametrizedAnnotation

A_MESSAGE = "A MESSAGE"
A_ROUTE = "/route"


class ParametrizedAnnotationTest(unittest.TestCase):

    def setUp(self):
        self.registry = Registry()

    def test_whenUsingParametrizedAnnotation_thenWrappedObjectsAreSavedInTheRegistry(self):
        wrapped_function = self.registry.get_annotated_in_package(myAnnotation, "")[0].registered

        self.assertEqual(some_wrapped_function, wrapped_function)

    def test_whenUsingParametrizedAnnotation_thenParametersAreStoredInTheRegistration(self):
        registration = self.registry.get_annotated_in_package(myAnnotation, "")[0]

        self.assertEqual(A_MESSAGE, registration.arguments['message'])
        self.assertFalse("value" in registration.arguments)

    def test_givenImplicitParameterName_whenUsingParametrizedAnnotation_thenParameterIsSavedAsTheDeclaredParameterName(self):
        registration = self.registry.get_annotated_in_package(implicit_parameter_annotation, "")[0]

        self.assertEqual(A_MESSAGE, registration.arguments['path'])

    def test_givenKeywordOnlyArguments_whenUsingParametrizedAnnotation_thenParametersAreSavedInTheRegistration(self):
        registration = self.registry.get_annotated_in_package(multipleParameterAnnotation, "")[0]

        self.assertEqual(A_MESSAGE, registration.arguments['message'])
        self.assertEqual(A_ROUTE, registration.arguments['route'])

    def test_givenMissingParameters_whenUsingOptionalArguments_thenPassNoneInstead(self):
        registration = self.registry.get_annotated_in_package(annotationWithOptionalParameters, "")[0]

        self.assertEqual(A_MESSAGE, registration.arguments['mandatory_parameter'])
        self.assertIsNone(registration.arguments['optional_parameter'])


@ParametrizedAnnotation
def myAnnotation(message: str) -> Callable:
    def wrapper(wrapped: Callable) -> "foo":
        return wrapped

    return wrapper


@myAnnotation(message=A_MESSAGE)
def some_wrapped_function():
    return "something"


@ParametrizedAnnotation
def implicit_parameter_annotation(path: str) -> Callable:
    def wrapper(auie):
        return auie

    return wrapper


@implicit_parameter_annotation(A_MESSAGE)
def implicit_wrapped_function():
    return "something"


@ParametrizedAnnotation
def multipleParameterAnnotation(*, message: str, route: str):
    return lambda x: x


@multipleParameterAnnotation(route=A_ROUTE, message=A_MESSAGE)
class AnnotatedWithMultipleParameters(object):
    pass


@ParametrizedAnnotation
def annotationWithOptionalParameters(optional_parameter: Optional[str], mandatory_parameter: str):
    return lambda x: x

@annotationWithOptionalParameters(mandatory_parameter=A_MESSAGE)
class AnnotatedWithOptionalParameters(object):
    pass

if __name__ == '__main__':
    unittest.main()



import sys

from jivago.lang.annotations import Override
from jivago.lang.stream import Stream
from jivago.serialization.serialization_strategy import SerializationStrategy


class NamedTupleSerializationStrategy(SerializationStrategy):

    def __init__(self, serializer: "Serializer"):
        self.serializer = serializer

    @Override
    def can_handle_serialization(self, obj) -> bool:
        declared_type = type(obj)
        if sys.version_info[0:2] == (3, 6):
            return hasattr(declared_type, "__bases__") \
                   and tuple in declared_type.__bases__ \
                   and hasattr(declared_type, "_field_types")

        return hasattr(declared_type, "__bases__") and tuple in declared_type.__bases__

    @Override
    def serialize(self, obj) -> object:
        return Stream(type(obj).__annotations__)\
            .map(lambda k: (k, self.serializer.serialize(obj.__getattribute__(k)))) \
            .toDict()



import argparse
import pkgutil
from typing import List


def generate_static_imports(root_app_module: str, outputfile: str):
    """Create a .py file containing all usually automatically imported packages as import my.app.package statements.
    Suitable for bundling the JivagoApplication using PyInstaller.

    :param root_app_module: Fully qualified import name for the app root.
    :param outputfile: The output .py file."""

    packages = __import_package_recursive(__import__(root_app_module, fromlist="dummy-str1"))

    with open(outputfile, 'w') as f:
        f.writelines([f"import {x}\n" for x in packages])


def __import_package_recursive(package) -> List[str]:
    prefix = package.__name__ + "."
    packages = []
    for importer, modname, ispkg in pkgutil.iter_modules(package.__path__, prefix):
        packages.append(modname)
        module = __import__(modname, fromlist="dummy-str1")
        if ispkg:
            packages.extend(__import_package_recursive(module))
    return packages


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description="""Create a .py file containing all usually automatically imported packages as import my.app.package statements. 
        Suitable for bundling the JivagoApplication using PyInstaller.""")
    parser.add_argument("--root-app-module", type=str, help="Fully qualified import name for the app root.",
                        dest="root", required=True)
    parser.add_argument("--outputfile", type=str, help="The output .py file.", dest="output", required=True)
    args = parser.parse_args()

    generate_static_imports(args.root, args.output)



from jivago.lang.annotations import Override, Inject
from jivago.wsgi.filter.filter import Filter
from jivago.wsgi.filter.filter_chain import FilterChain
from jivago.wsgi.request.request import Request
from jivago.wsgi.request.response import Response
from jivago.wsgi.routing.cors.cors_handler import CorsHandler


class CorsHeadersInjectionFilter(Filter):

    @Inject
    def __init__(self, cors_handler: CorsHandler):
        self.cors_handler = cors_handler

    @Override
    def doFilter(self, request: Request, response: Response, chain: FilterChain):
        chain.doFilter(request, response)

        if response.status != 404:
            self.cors_handler.inject_cors_headers(request.path, response.headers)



import sys
from typing import NamedTuple, Type

from jivago.lang.annotations import Override
from jivago.lang.stream import Stream
from jivago.serialization.deserialization_strategy import DeserializationStrategy
from jivago.wsgi.invocation.incorrect_attribute_type_exception import IncorrectAttributeTypeException


class NamedTupleDeserializationStrategy(DeserializationStrategy):

    def __init__(self, deserializer: "Deserializer"):
        self.deserializer = deserializer

    @Override
    def can_handle_deserialization(self, declared_type: type) -> bool:
        if sys.version_info[0:2] == (3, 6):
            return hasattr(declared_type, "__bases__") \
                   and tuple in declared_type.__bases__ \
                   and hasattr(declared_type, "_field_types")

        return hasattr(declared_type, "__bases__") and tuple in declared_type.__bases__

    @Override
    def deserialize(self, obj, declared_type: Type[NamedTuple]) -> NamedTuple:
        if sys.version_info[0:2] == (3, 6):
            if isinstance(obj, dict):
                parameters = {}
                for name, clazz in declared_type._field_types.items():
                    parameters[name] = self.deserializer.deserialize(obj[name], clazz)
                return declared_type(**parameters)
            elif isinstance(obj, list):
                return declared_type(*obj)
            raise IncorrectAttributeTypeException()

        if isinstance(obj, dict):
            attributes = declared_type.__annotations__
            parameters = Stream(attributes.items()) \
                .map(lambda name, attribute_type:
                     self.deserializer.deserialize(obj.get(name), attribute_type)) \
                .toList()
        elif isinstance(obj, list):
            parameters = obj
        else:
            raise IncorrectAttributeTypeException()
        return declared_type(*parameters)



import unittest

from jivago.inject.scope.scope_cache import ComponentNotHandledByScopeException, UninstantiatedObjectException
from jivago.inject.scope.singleton_scope_cache import SingletonScopeCache
from jivago.lang.registry import Annotation


class SingletonScopeCacheTest(unittest.TestCase):

    def setUp(self):
        self.scope = SingletonScopeCache(SomeScope, [SomeScopedComponent])

    def test_whenCheckingHandlesComponent_thenChecksWhetherTheComponentIsInTheScopedComponentsList(self):
        scoped = self.scope.handles_component(SomeScopedComponent)

        self.assertTrue(scoped)

    def test_givenUnscopedComponent_whenCheckingHandlesComponent_thenReturnFalse(self):
        scoped = self.scope.handles_component(UnscopedComponent)

        self.assertFalse(scoped)

    def test_givenUnhandledComponent_whenStoringObjectInstance_thenThrowComponentNotHandledByScopeException(self):
        with self.assertRaises(ComponentNotHandledByScopeException):
            self.scope.store(UnscopedComponent, UnscopedComponent())

    def test_whenGettingStoredObjectInstance_thenReturnSavedInstance(self):
        component = SomeScopedComponent()
        self.scope.store(SomeScopedComponent, component)

        object_instance = self.scope.get(SomeScopedComponent)

        self.assertEqual(component, object_instance)

    def test_givenAComponentWhichHasNeverBeenStored_whenGettingInstance_thenThrowUninstantiatedObjectException(self):
        with self.assertRaises(UninstantiatedObjectException):
            self.scope.get(SomeScopedComponent)


@Annotation
def SomeScope(x):
    return x


class SomeScopedComponent(object):
    pass


class UnscopedComponent(object):
    pass


if __name__ == '__main__':
    unittest.main()



import threading
import unittest

from jivago.inject.scope.request_scope_cache import RequestScopeCache
from jivago.inject.scope.scope_cache import UninstantiatedObjectException, ComponentNotHandledByScopeException


class RequestScopeCacheTest(unittest.TestCase):

    def setUp(self):
        self.scope = RequestScopeCache([SomeScopedComponent])

    def test_whenCheckingHandlesComponent_thenChecksWhetherTheComponentIsInTheScopedComponentsList(self):
        scoped = self.scope.handles_component(SomeScopedComponent)

        self.assertTrue(scoped)

    def test_givenUnscopedComponent_whenCheckingHandlesComponent_thenReturnFalse(self):
        scoped = self.scope.handles_component(UnscopedComponent)

        self.assertFalse(scoped)

    def test_givenUnhandledComponent_whenStoringObjectInstance_thenThrowComponentNotHandledByScopeException(self):
        with self.assertRaises(ComponentNotHandledByScopeException):
            self.scope.store(UnscopedComponent, UnscopedComponent())

    def test_whenStoringAComponent_thenItCanBeRetrieved(self):
        instance = SomeScopedComponent()
        self.scope.store(SomeScopedComponent, instance)

        stored = self.scope.get(SomeScopedComponent)

        self.assertEqual(instance, stored)

    def test_givenStoredComponentFromOtherThread_whenGettingComponent_thenRaiseException(self):
        self.scope.store(SomeScopedComponent, SomeScopedComponent())

        def retrieve_and_assert():
            with self.assertRaises(UninstantiatedObjectException):
                self.scope.get(SomeScopedComponent)

        threading.Thread(target=retrieve_and_assert).start()

    def test_whenClearing_thenClearsTheStorageForTheCurrentThread(self):
        self.scope.store(SomeScopedComponent, SomeScopedComponent())

        self.scope.clear()

        with self.assertRaises(UninstantiatedObjectException):
            self.scope.get(SomeScopedComponent)


class SomeScopedComponent(object):
    pass


class UnscopedComponent(object):
    pass



class SerializationException(Exception):
    pass


class NoMatchingDeserializationStrategyException(SerializationException):
    def __init__(self, clazz: type):
        super().__init__(self,
                         f"Could not find an appropriate deserialization strategy for type {clazz}. Are you missing the '@Serializable' annotation?")



import io


class StreamingRequestBody(object):

    def __init__(self, content: io.RawIOBase):
        self.content = content

    def read(self, n: int = 1) -> bytes:
        return self.content.read(n)

    def readall(self) -> bytes:
        return self.content.readall()

    def readinto(self, out: bytearray) -> int:
        return self.content.readinto(out)



import unittest

from jivago.inject.annoted_class_binder import AnnotatedClassBinder
from jivago.inject.service_locator import ServiceLocator
from jivago.lang.registry import Annotation, Registry


class AnnotatedClassBinderTest(unittest.TestCase):

    def setUp(self):
        self.service_locator = ServiceLocator()
        self.registry = Registry()
        self.binder = AnnotatedClassBinder("", self.registry, _binding_annotation)

    def test_whenBinding_thenBindAllAnnotatedClassesToThemselves(self):
        self.binder.bind(self.service_locator)

        instance = self.service_locator.get(SomeComponent)
        self.assertIsInstance(instance, SomeComponent)


_binding_annotation = Annotation()


@_binding_annotation
class SomeComponent(object):
    pass



import unittest

from jivago.lang.registry import Registry
from jivago.wsgi.annotations import Path, Resource
from jivago.wsgi.methods import GET
from jivago.wsgi.routing.table.reflective_routing_table import ReflectiveRoutingTable


class ReflectiveRoutingTableTests(unittest.TestCase):
    def test_method_with_multiple_registered_paths(self):
        table = ReflectiveRoutingTable(REGISTRY, REGISTRY.get_annotated_in_package(Resource, __name__))

        a = table.get_route_registrations("/a")
        b = table.get_route_registrations("/b")

        self.assertEqual(a[0].routeFunction, b[0].routeFunction)


REGISTRY = Registry()


@Resource("/")
class MyResource(object):

    @GET
    @Path("a")
    @Path("b")
    def method(self):
        pass



import unittest

from jivago.lang.annotations import Serializable
from jivago.serialization.object_mapper import ObjectMapper
from jivago.serialization.serialization_exception import SerializationException


class ObjectMapperTest(unittest.TestCase):

    def setUp(self):
        self.object_mapper = ObjectMapper()

    def test_givenUnregisteredType_whenDeserializing_thenThrowException(self):
        with self.assertRaises(SerializationException):
            self.object_mapper.deserialize('{ "name": "foo" }', A_Dto)

    def test_givenRegisteredType_whenSerializing_thenDeserializeCorrectly(self):
        dto = self.object_mapper.deserialize('{ "name": "foo" }', A_RegisteredDto)

        self.assertEqual("foo", dto.name)

    def test_givenAllowUnregisteredTypes_whenDeserializing_thenIgnoreClassRegistrationStatus(self):
        self.object_mapper = ObjectMapper(allow_unregistered_types=True)

        dto = self.object_mapper.deserialize('{ "name": "foo" }', A_Dto)

        self.assertEqual("foo", dto.name)

    def test_givenAllowUnregisteredTypes_whenSerializing_thenIgnoreClassRegistrationStatus(self):
        self.object_mapper = ObjectMapper(allow_unregistered_types=True)
        dto = A_Dto()
        dto.name = "foo"

        json_str = self.object_mapper.serialize(dto)

        self.assertEqual('{"name": "foo"}', json_str)


class A_Dto(object):
    name: str


@Serializable
class A_RegisteredDto(object):
    name: str



import unittest
import time
from datetime import datetime
from unittest import mock

from jivago.inject.service_locator import ServiceLocator
from jivago.lang.annotations import Override
from jivago.lang.runnable import Runnable
from jivago.scheduling.schedule import Schedule
from jivago.scheduling.scheduled_task_runner import ScheduledTaskRunner


class ScheduledTaskRunnerTests(unittest.TestCase):

    def setUp(self):
        self.service_locator = ServiceLocator()
        self.service_locator.bind(SomeScheduledTask, SomeScheduledTask)
        self.service_locator.bind(SomeCrashingTask, SomeCrashingTask)
        self.service_locator.bind(SomeTaskCrashingOnCleanup, SomeTaskCrashingOnCleanup)
        self.schedule_mock: Schedule = mock.create_autospec(Schedule)
        self.schedule_mock.next_start_time.return_value = datetime.utcnow()

    def test_run_scheduled_task(self):
        self.runner = ScheduledTaskRunner(SomeScheduledTask, self.schedule_mock, self.service_locator)

        self.runner.start()

        self.assertTrue(SomeScheduledTask.was_called)

    def test_givenUncaughtException_whenExecuting_shouldNotCrash(self):
        self.runner = ScheduledTaskRunner(SomeCrashingTask, self.schedule_mock, self.service_locator)

        self.runner.start()
        time.sleep(0.05)

        self.assertTrue(SomeCrashingTask.times_called > 1)

    def test_givenUncaughtException_whenCleaningUp_shouldNotCrash(self):
        self.runner = ScheduledTaskRunner(SomeTaskCrashingOnCleanup, self.schedule_mock, self.service_locator)

        self.runner.start()
        self.runner.stop()

        self.assertTrue(SomeTaskCrashingOnCleanup.run_called)
        self.assertTrue(SomeTaskCrashingOnCleanup.cleanup_called)

    def tearDown(self):
        self.runner.stop()


class SomeScheduledTask(Runnable):
    was_called = False

    @Override
    def run(self):
        SomeScheduledTask.was_called = True


class SomeCrashingTask(Runnable):
    times_called = 0

    @Override
    def run(self):
        SomeCrashingTask.times_called += 1

        if SomeCrashingTask.times_called == 1:
            raise Exception("Error!!")


class SomeTaskCrashingOnCleanup(Runnable):
    cleanup_called = False
    run_called = False

    def run(self):
        SomeTaskCrashingOnCleanup.run_called = True

    def cleanup(self):
        SomeTaskCrashingOnCleanup.cleanup_called = True
        raise Exception("Error!")



from typing import Callable, TypeVar

from jivago.lang.registry import Annotation

T_OVERRIDABLE = TypeVar("T_OVERRIDABLE", Callable, property)

def Override(fun: T_OVERRIDABLE) -> T_OVERRIDABLE:
    return fun


@Annotation
def Inject(fun: Callable) -> Callable:
    return fun


@Annotation
def Serializable(clazz: type) -> type:
    return clazz


@Annotation
def BackgroundWorker(clazz: type) -> type:
    return clazz



from typing import List, Optional

from jivago.config.router.cors_rule import CorsRule
from jivago.lang.stream import Stream
from jivago.wsgi.request.headers import Headers
from jivago.wsgi.routing.cors.cors_preflight_request_handler import CorsPreflightRequestHandler
from jivago.wsgi.routing.cors.no_matching_cors_rule_exception import NoMatchingCorsRuleException


class CorsHandler(object):

    def __init__(self, cors_rules: List[CorsRule]):
        self.cors_rules = cors_rules

    def create_cors_preflight_handler(self, path: str) -> CorsPreflightRequestHandler:
        highest_priority_rule = self._find_highest_priority_rule(path)

        if highest_priority_rule:
            return CorsPreflightRequestHandler(highest_priority_rule.cors_headers)

        raise NoMatchingCorsRuleException()

    def _find_highest_priority_rule(self, path: str) -> Optional[CorsRule]:
        return Stream(self.cors_rules) \
            .filter(lambda rule: rule.matches(path)) \
            .reduce(None, lambda highest, e: e if highest is None or e.takes_precedence_over(highest) else highest)

    def inject_cors_headers(self, path: str, response_headers: Headers) -> None:
        rule = self._find_highest_priority_rule(path)

        if rule:
            rule.inject_headers(response_headers)



import unittest
from typing import List

from jivago.inject.annotation import Singleton
from jivago.inject.scope.singleton_scope_cache import SingletonScopeCache
from jivago.inject.service_locator import ServiceLocator, InstantiationException, NonInjectableConstructorException
from jivago.lang.annotations import Inject
from jivago.lang.registry import Registry
from jivago.lang.stream import Stream


class ServiceLocatorTest(unittest.TestCase):
    A_LITERAL_OBJECT = "A Message"

    def setUp(self):
        self.serviceLocator = ServiceLocator(Registry())

    def test_givenInexistentComponent_whenGettingComponent_thenThrowInstantiationException(self):
        with self.assertRaises(InstantiationException):
            self.serviceLocator.get(SomeClass)

    def test_givenLiteralComponent_whenGettingComponent_thenReturnsLiteralWithoutInstantiation(self):
        self.serviceLocator.bind(str, self.A_LITERAL_OBJECT)

        component = self.serviceLocator.get(str)

        self.assertEqual(self.A_LITERAL_OBJECT, component)

    def test_givenInstantiatableComponent_whenGettingComponent_thenReturnInstantiatedObject(self):
        self.serviceLocator.bind(SomeClass, SomeClass)

        component = self.serviceLocator.get(SomeClass)

        self.assertIsInstance(component, SomeClass)

    def test_givenBoundAbstraction_whenGettingChildComponent_thenReturnChildComponent(self):
        self.serviceLocator.bind(SomeClass, SomeChildClass)

        component = self.serviceLocator.get(SomeClass)

        self.assertIsInstance(component, SomeChildClass)

    def test_givenConstructorWithParameters_whenGettingComponent_thenRecursivelyInstantiateConstructorParameters(self):
        self.serviceLocator.bind(SomeClassWithParameters, SomeClassWithParameters)
        self.serviceLocator.bind(SomeClass, SomeClass)

        component = self.serviceLocator.get(SomeClassWithParameters)

        self.assertIsInstance(component, SomeClassWithParameters)
        self.assertIsInstance(component.someObject, SomeClass)

    def test_givenProviderMethodWithParameters_whenGettingComponent_thenCallProviderMethodWithRecursivelyInstantiatedParameters(
            self):
        self.serviceLocator.bind(SomeChildClass, provider_function_with_parameters)
        self.serviceLocator.bind(SomeClass, SomeClass)

        component = self.serviceLocator.get(SomeChildClass)

        self.assertIsInstance(component, SomeChildClass)

    def test_givenScopedComponent_whenGettingComponentTwice_thenReturnTheSameComponentTwice(self):
        scope_cache = SingletonScopeCache(Singleton, [SomeClass])
        self.serviceLocator.register_scope(scope_cache)
        self.serviceLocator.bind(SomeClass, SomeClass)

        expected = self.serviceLocator.get(SomeClass)
        result = self.serviceLocator.get(SomeClass)

        self.assertIsInstance(expected, SomeClass)
        self.assertEqual(expected, result)

    def test_givenClassWithNonInjectableConstructor_whenGettingComponent_thenThrowNonInjectableConstructorException(
            self):
        self.serviceLocator.bind(SomeClassWithANonInjectableConstructor, SomeClassWithANonInjectableConstructor)

        with self.assertRaises(NonInjectableConstructorException):
            self.serviceLocator.get(SomeClassWithANonInjectableConstructor)

    def test_givenNonInjectableConstructorWithOnlyOneParameter_whenGettingComponent_thenInstantiateTheComponentAnyway(
            self):
        self.serviceLocator.bind(ClassWithImplicitlyInjectableConstructor, ClassWithImplicitlyInjectableConstructor)

        component = self.serviceLocator.get(ClassWithImplicitlyInjectableConstructor)

        self.assertIsInstance(component, ClassWithImplicitlyInjectableConstructor)

    def test_givenAConstructorWhichContainsAFunctionDefinition_whenGettingComponent_thenComponentIsInstantiatedWithoutInterferingWithTheFunction(
            self):
        self.serviceLocator.bind(ClassWhichContainsAFunctionInItsConstructor,
                                 ClassWhichContainsAFunctionInItsConstructor)
        self.serviceLocator.bind(SomeClass, SomeClass)

        component = self.serviceLocator.get(ClassWhichContainsAFunctionInItsConstructor)

        self.assertIsInstance(component, ClassWhichContainsAFunctionInItsConstructor)

    def test_givenAProviderFunctionWithAnotherFunctionDefinedInsideIt_whenGettingComponent_thenComponentIsInstantiatedWithoutInterferingWithTheInnerFunction(
            self):
        self.serviceLocator.bind(SomeChildClass, function_with_another_function_defined_inside_it)
        self.serviceLocator.bind(SomeClassWithParameters, SomeClassWithParameters)
        self.serviceLocator.bind(SomeClass, SomeClass)

        component = self.serviceLocator.get(SomeChildClass)

        self.assertIsInstance(component, SomeChildClass)

    def test_givenList_whenGettingComponent_thenReturnAllInstancesOfClass(self):
        self.serviceLocator.bind(SomeClass, SomeClass)
        self.serviceLocator.bind(SomeChildClass, SomeChildClass)

        components = self.serviceLocator.get(List[SomeClass])

        self.assertEqual(2, len(components))
        self.assertTrue(Stream(components).allMatch(lambda c: isinstance(c, SomeClass)))

    def test_whenManuallyInstantiatingAnInjectable_thenPassSelfVariableAndDoNotCrash(self):
        # Injectable should not interfere with the base class
        # fix crash on version 0.0.9
        obj = SomeClassWithParameters(None)

        self.assertIsInstance(obj, SomeClassWithParameters)


class SomeClass(object):
    pass


class SomeChildClass(SomeClass):
    pass


class SomeClassWithParameters(object):
    @Inject
    def __init__(self, some_object: SomeClass):
        self.someObject = some_object


class SomeClassWithANonInjectableConstructor(object):
    def __init__(self, some_object: SomeClass):
        pass


class ClassWithImplicitlyInjectableConstructor(object):
    def __init__(self):
        self.value = 5


def provider_function_with_parameters(parameter: SomeClass) -> SomeChildClass:
    return SomeChildClass()


class ClassWhichContainsAFunctionInItsConstructor(object):

    @Inject
    def __init__(self, some_class: SomeClass):
        someVariable = 123
        self.someMessage = "hello"

        def wrapper(some_class=some_class) -> int:
            return 5

        self.someObject = wrapper(some_class)


def function_with_another_function_defined_inside_it(parameter: SomeClass) -> SomeChildClass:
    someVariable = 10

    def someWrapperFunction(param: SomeClassWithParameters) -> SomeChildClass:
        return SomeChildClass()

    someVariable += 10
    return someWrapperFunction(parameter)


if __name__ == "__main__":
    unittest.main()



class RenderedView(object):

    def __init__(self, view_file: str, data,*, content_type: str = "text/html"):
        self.view_file = view_file
        self.data = data
        self.content_type = content_type



from jivago.inject.scope.request_scope_cache import RequestScopeCache
from jivago.lang.annotations import Override, Inject
from jivago.wsgi.filter.filter import Filter
from jivago.wsgi.filter.filter_chain import FilterChain
from jivago.wsgi.request.request import Request
from jivago.wsgi.request.response import Response


class RequestScopeFilter(Filter):

    @Inject
    def __init__(self, request_scope: RequestScopeCache):
        self.request_scope = request_scope

    @Override
    def doFilter(self, request: Request, response: Response, chain: FilterChain):
        try:
            chain.doFilter(request, response)
        finally:
            self.request_scope.clear()



import unittest
from unittest import mock
from unittest.mock import Mock, call

from jivago.inject.scope.request_scope_cache import RequestScopeCache
from jivago.wsgi.filter.filter_chain import FilterChain
from jivago.wsgi.filter.system_filters.request_scope_filter import RequestScopeFilter
from test_utils.request_builder import RequestBuilder
from test_utils.response_builder import ResponseBuilder


class RequestScopeFilterTest(unittest.TestCase):

    def setUp(self):
        self.request_scope_mock: RequestScopeCache = mock.create_autospec(RequestScopeCache)
        self.filter_chain_mock: FilterChain = mock.create_autospec(FilterChain)
        self.filter = RequestScopeFilter(self.request_scope_mock)
        self.mock_container = Mock()
        self.mock_container.filter_chain, self.mock_container.request_scope = self.filter_chain_mock, self.request_scope_mock

    def test_whenApplyingFilter_shouldClearScopeCacheAfterInvokingFilterChain(self):
        self.filter.doFilter(SOME_REQUEST, SOME_RESPONSE, self.filter_chain_mock)

        self.mock_container.assert_has_calls(
            [call.filter_chain.doFilter(SOME_REQUEST, SOME_RESPONSE),
             call.request_scope.clear()])

    def test_givenUncaughtException_whenApplyingFilter_shouldClearCacheAndRethrow(self):
        self.filter_chain_mock.doFilter.side_effect = MyException()

        with self.assertRaises(MyException):
            self.filter.doFilter(SOME_REQUEST, SOME_RESPONSE, self.filter_chain_mock)

        self.request_scope_mock.clear.assert_called()


SOME_RESPONSE = ResponseBuilder().build()
SOME_REQUEST = RequestBuilder().build()


class MyException(Exception):
    pass



import unittest

from jivago.wsgi.invocation.missing_route_invocation_argument import MissingRouteInvocationArgument
from jivago.wsgi.invocation.parameter_selection.streaming_request_body_parameter_selector import \
    StreamingRequestBodyParameterSelector
from jivago.wsgi.request.streaming_request_body import StreamingRequestBody
from test_utils.request_builder import RequestBuilder


class StreamingRequestBodyParameterSelectorTest(unittest.TestCase):

    def setUp(self):
        self.selector = StreamingRequestBodyParameterSelector()

    def test_shouldMatchStreamingRequestBodyType(self):
        matches = self.selector.matches(StreamingRequestBody)

        self.assertTrue(matches)

    def test_givenStreamingRequestBody_whenFormattingParameter_thenReturnResponseBody(self):
        body = StreamingRequestBody("dummy")
        request = RequestBuilder().body(body).build()

        parameter = self.selector.format_parameter("param name", StreamingRequestBody, request)

        self.assertEqual(body, parameter)

    def test_givenIncorrectRequestBody_whenFormattingParameter_thenRaiseException(self):
        request = RequestBuilder().build()

        with self.assertRaises(MissingRouteInvocationArgument):
            self.selector.format_parameter("param name", StreamingRequestBody, request)



import unittest

from jivago.serialization.deserialization.list_deserialization_strategy import ListDeserializationStrategy
from jivago.wsgi.invocation.incorrect_attribute_type_exception import IncorrectAttributeTypeException


class ListDeserializationStrategyTests(unittest.TestCase):

    def setUp(self):
        self.strategy = ListDeserializationStrategy()

    def test_deserializes_untyped_list(self):
        result = self.strategy.deserialize([1, 2, 3], list)

        self.assertEqual([1, 2, 3], result)

    def test_givenIncorrectType_whenDeserializing_shouldRaiseException(self):
        with self.assertRaises(IncorrectAttributeTypeException):
            self.strategy.deserialize("", list)

    def test_canHandleListDeserializationOnly(self):
        can_handle_list = self.strategy.can_handle_deserialization(list)
        can_handle_other = self.strategy.can_handle_deserialization(dict)

        self.assertTrue(can_handle_list)
        self.assertFalse(can_handle_other)



import unittest
from unittest import mock

from jivago.wsgi.filter.filter_chain import FilterChain
from jivago.wsgi.request.response import Response
from jivago.wsgi.routing.cors.cors_handler import CorsHandler
from jivago.wsgi.routing.cors.cors_headers_injection_filter import CorsHeadersInjectionFilter
from test_utils.request_builder import RequestBuilder
from test_utils.response_builder import ResponseBuilder


class CorsHeadersInjectionFilterTests(unittest.TestCase):

    def setUp(self):
        self.cors_handler_mock: CorsHandler = mock.create_autospec(CorsHandler)
        self.filter_chain_mock: FilterChain = mock.create_autospec(FilterChain)
        self.filter = CorsHeadersInjectionFilter(self.cors_handler_mock)
        self.request = RequestBuilder().build()

    def test_shouldInjectCorsHeadersInResponse(self):
        response = Response.empty()

        self.filter.doFilter(self.request, response, self.filter_chain_mock)

        self.cors_handler_mock.inject_cors_headers.assert_called_with(self.request.path, response.headers)

    def test_given404Response_shouldNotInjectCorsHeaders(self):
        response = ResponseBuilder().status(404).build()

        self.filter.doFilter(self.request, response, self.filter_chain_mock)

        self.cors_handler_mock.inject_cors_headers.assert_not_called()



from jivago.lang.annotations import Override
from jivago.wsgi.filter.filter import Filter
from jivago.wsgi.filter.filter_chain import FilterChain
from jivago.wsgi.request.request import Request
from jivago.wsgi.request.response import Response
from jivago.wsgi.request.streaming_response_body import StreamingResponseBody


class StreamingResponseHeaderFilter(Filter):

    @Override
    def doFilter(self, request: Request, response: Response, chain: FilterChain):
        chain.doFilter(request, response)

        if isinstance(response.body, StreamingResponseBody):
            response.headers["Transfer-Encoding"] = "chunked"



import unittest

from jivago.inject.annotation import Provider
from jivago.inject.exception.undefined_return_provider_function import UndefinedReturnProviderFunction
from jivago.inject.provider_binder import ProviderBinder
from jivago.inject.service_locator import ServiceLocator
from jivago.lang.registry import Registry


class ProviderBinderTest(unittest.TestCase):

    def setUp(self):
        self.serviceLocator = ServiceLocator()
        self.result = []
        self.serviceLocator.bind = lambda interface, implementation: self.result.append((interface, implementation))

    def test_givenProviderFunctionWithReturnType_whenInitializingServiceLocator_thenFunctionIsBoundByReturnType(self):
        binder = ProviderBinder("", Registry())

        binder.bind(self.serviceLocator)

        self.assertEqual((SomeClass, provider_function), self.result[0])

    def test_givenProviderFunctionWithoutAReturnType_whenInitializingProviderBinder_thenThrowUndefinedReturnProviderFunctionException(
            self):
        @Provider
        def provider_function_without_a_defined_return_type():
            return SomeClass()

        binder = ProviderBinder("", Registry())

        with self.assertRaises(UndefinedReturnProviderFunction):
            binder.bind(self.serviceLocator)


class SomeClass(object):
    pass


@Provider
def provider_function() -> SomeClass:
    return SomeClass()


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest import mock

from jivago.wsgi.filter.filter_chain import FilterChain
from jivago.wsgi.filter.system_filters.streaming_response_header_filter import StreamingResponseHeaderFilter
from jivago.wsgi.request.streaming_response_body import StreamingResponseBody
from test_utils.request_builder import RequestBuilder
from test_utils.response_builder import ResponseBuilder


class StreamingResponseHeaderFilterTests(unittest.TestCase):

    def setUp(self):
        self.filter = StreamingResponseHeaderFilter()
        self.filterChainMock: FilterChain = mock.create_autospec(FilterChain)

    def test_givenStreamingResponseBody_whenApplyingFilter_ShouldAddTransferEncodingHeader(self):
        response = ResponseBuilder().body(StreamingResponseBody([])).build()

        self.filter.doFilter(RequestBuilder().build(), response, self.filterChainMock)

        self.assertEqual("chunked", response.headers["Transfer-Encoding"])

    def test_givenAnyResponse_whenApplyingFilter_ShouldNotAddTransferEncodingHeader(self):
        response = ResponseBuilder().body("some body").build()

        self.filter.doFilter(RequestBuilder().build(), response, self.filterChainMock)

        self.assertIsNone(response.headers["Transfer-Encoding"])



from jivago.templating.template_filter import TemplateFilter
from jivago.wsgi.filter.system_filters.banner_filter import BannerFilter
from jivago.wsgi.filter.system_filters.body_serialization_filter import BodySerializationFilter
from jivago.wsgi.filter.system_filters.error_handling.application_exception_filter import ApplicationExceptionFilter
from jivago.wsgi.filter.system_filters.error_handling.unknown_exception_filter import UnknownExceptionFilter
from jivago.wsgi.filter.system_filters.request_scope_filter import RequestScopeFilter
from jivago.wsgi.filter.system_filters.streaming_response_header_filter import StreamingResponseHeaderFilter
from jivago.wsgi.request.http_form_deserialization_filter import HttpFormDeserializationFilter
from jivago.wsgi.request.json_serialization_filter import JsonSerializationFilter
from jivago.wsgi.routing.cors.cors_headers_injection_filter import CorsHeadersInjectionFilter

JIVAGO_DEFAULT_FILTERS = [BannerFilter,
                          CorsHeadersInjectionFilter,
                          UnknownExceptionFilter,
                          TemplateFilter,
                          JsonSerializationFilter,
                          HttpFormDeserializationFilter,
                          StreamingResponseHeaderFilter,
                          BodySerializationFilter,
                          ApplicationExceptionFilter,
                          RequestScopeFilter]

# Required filters for HTTP OPTIONS requests.
JIVAGO_DEFAULT_OPTIONS_FILTERS = [BannerFilter,
                                  CorsHeadersInjectionFilter,
                                  UnknownExceptionFilter,
                                  JsonSerializationFilter,
                                  BodySerializationFilter,
                                  ApplicationExceptionFilter,
                                  RequestScopeFilter]



from typing import List

from jivago.inject.scope.scope_cache import ScopeCache, UninstantiatedObjectException, \
    ComponentNotHandledByScopeException
from jivago.lang.stream import Stream


class SingletonScopeCache(ScopeCache):

    def __init__(self, name: str, scoped_components: List[type]):
        """
        :param name: human-readable name
        :param scoped_components: components managed by this scope
        """
        super().__init__(name)
        self.scoped_components = Stream(scoped_components).map(lambda clazz: (clazz, None)).toDict()

    def handles_component(self, component: type) -> bool:
        return component in self.scoped_components.keys()

    def is_stored(self, component: type) -> bool:
        return self.scoped_components[component] is not None

    def get(self, component: type) -> object:
        stored_component = self.scoped_components.get(component)
        if stored_component is None:
            raise UninstantiatedObjectException(component)
        return stored_component

    def store(self, component: type, instance: object):
        if component not in self.scoped_components.keys():
            raise ComponentNotHandledByScopeException(component)
        self.scoped_components[component] = instance



class ScopeCache(object):

    def __init__(self, name: str):
        """
        :param name: human-readable name
        :param scoped_components: components managed by this scope
        """
        self.name = name

    def handles_component(self, component: type) -> bool:
        raise NotImplementedError

    def is_stored(self, component: type) -> bool:
        raise NotImplementedError

    def get(self, component: type) -> object:
        raise NotImplementedError

    def store(self, component: type, instance: object):
        raise NotImplementedError


class ComponentNotHandledByScopeException(Exception):
    pass


class UninstantiatedObjectException(Exception):
    pass



import threading
from typing import List

from jivago.inject.annotation import RequestScoped
from jivago.inject.scope.scope_cache import ScopeCache, UninstantiatedObjectException, \
    ComponentNotHandledByScopeException
from jivago.lang.annotations import Override


class RequestScopeCache(ScopeCache):

    @Override
    def __init__(self, scoped_components: List[type]):
        super().__init__(str(RequestScoped))
        self.scoped_components = scoped_components
        self._storage = threading.local()

    def clear(self):
        self._storage.__dict__.clear()

    @Override
    def handles_component(self, component: type) -> bool:
        return component in self.scoped_components

    @Override
    def is_stored(self, component: type) -> bool:
        return hasattr(self._storage, component.__qualname__)

    @Override
    def get(self, component: type) -> object:
        if not self.is_stored(component):
            raise UninstantiatedObjectException(component)
        return getattr(self._storage, component.__qualname__)

    @Override
    def store(self, component: type, instance: object):
        if component not in self.scoped_components:
            raise ComponentNotHandledByScopeException(component)
        self._storage.__setattr__(component.__qualname__, instance)
